//
//  Camera.swift
//  EmojiArt
//
//  Created by CS193p Instructor on 5/24/21.
//

import SwiftUI

struct Camera: UIViewControllerRepresentable {
    var handlePickedImage: (UIImage?) -> Void//设备数据传递
    //相机是否可用
    static var isAvailable: Bool {
        //UIImagePickerController用于管理拍照、录制电影和从用户的媒体库中选择项目的系统接口（54：00）。
        UIImagePickerController.isSourceTypeAvailable(.camera)//查询设备是否支持使用指定的源类型选择媒体。
    }
    //创建自定义实例，用于将视图控制器的更改传递给swiftui界面的其他部分。
    func makeCoordinator() -> Coordinator {
        //返回给处理委托的协调员(class Coordinator)
        Coordinator(handlePickedImage: handlePickedImage)
    }
    //制作一个控制器
    func makeUIViewController(context: Context) -> UIImagePickerController {
        let picker = UIImagePickerController()//管理拍照、录制电影和从用户的媒体库中选择项目的系统接口。
        picker.sourceType = .camera//让picker控制器显示的选择器界面的类型为设备的内置摄像头。
        picker.allowsEditing = true//指示是否允许用户编辑选定的静态图像或电影（56:20）。
        picker.delegate = context.coordinator//picker的委托对象（拍照完成后委托给谁来处理:通过makeCoordinator把数据交给class Coordinator）。
        return picker
    }
    //更新控制器
    func updateUIViewController(_ uiViewController: UIImagePickerController, context: Context) {
        // 啥事儿也不做。写来就是为了符合协议（52:29）
    }
    //被委托的对象处理拍照完成后要做的事情（58：01）
    //NSObject是UIKit的基类，必须符合这个协议
    //UIImagePickerControllerDelegate实现才能与图像选择器接口交互的方法的协议
    //UINavigationControllerDelegate为UIImagePickerControllerDelegate的前提协议，啥事也不做
    class Coordinator: NSObject, UIImagePickerControllerDelegate, UINavigationControllerDelegate {
        var handlePickedImage: (UIImage?) -> Void //从makeCoordinator获取
        init(handlePickedImage: @escaping (UIImage?) -> Void) {
            self.handlePickedImage = handlePickedImage //使用Coordinator初始化这个闭包
        }
        //告诉委托用户取消了挑选操作（符合UIImagePickerControllerDelegate协议）。
        func imagePickerControllerDidCancel(_ picker: UIImagePickerController) {
            handlePickedImage(nil)//将handlePickedImage设置为nil
        }
        //告诉委托用户选择了静态图像或电影（符合UIImagePickerControllerDelegate协议）。
        func imagePickerController(_ picker: UIImagePickerController, didFinishPickingMediaWithInfo info: [UIImagePickerController.InfoKey : Any]) {
            handlePickedImage((info[.editedImage] ?? info[.originalImage]) as? UIImage)//将选择的照片转成UIImage并交给handlePickedImage
        }
    }
}
