//
//  iOS.swift
//  EmojiArt
//
//  Created by CS193p Instructor on 5/26/21.
//

import SwiftUI

// L16 iOS-specific code

extension UIImage {
    // L16 convenience var to turn a UIImage into a Data
    // L16 on iOS, it converts it to jpeg
    var imageData: Data? { jpegData(compressionQuality: 1.0) }
}

// 包含静态访问Pasteboard的结构体,以独立于平台的方式使用
struct Pasteboard {
    static var imageData: Data? {
        UIPasteboard.general.image?.imageData
    }
    static var imageURL: URL? {
        UIPasteboard.general.url?.imageURL
    }
}

// L16 extensions to View for platform-specific modifications

extension View {
    // L16 palette control button uses the default style on iOS
    func paletteControlButtonStyle() -> some View {
        self
    }
    
    // L16 no extra padding required on iOS in a popover
    func popoverPadding() -> some View {
        self
    }
    @ViewBuilder //此函数声明了一个不透明的返回类型，但在函数体中没有用于推断底层类型的返回语句
    //dismiss为一个Optional闭包
    func wrappedInNavigationViewToMakeDismissable(_ dismiss:(()->Void)?) -> some View {
        //非iPad，闭包代码存在的情况
        if UIDevice.current.userInterfaceIdiom != .pad, let dismiss = dismiss{
            NavigationView{
                self
                    .navigationBarTitleDisplayMode(.inline)//设置为小标题
                    .dismissable(dismiss)//学习二次调用扩展里的修改器
                //下面的.toolbar也能正常工作
                //.toolbar {
                //    ToolbarItem(placement: .cancellationAction) {
                //       Button("关闭"){ dismiss()}
                //    }
                // }
            }
            .navigationViewStyle(StackNavigationViewStyle())//(22:06)一种由一次只显示一个顶部视图的视图堆栈表示的导航视图样式。
            //当在NavigationView中有足够的水平空间时(iPhone横屏、iPad、Mac)，SwiftUI默认会自动放置左、右2个视图。
        }else{
            self //不修改返回
        }
    }
    @ViewBuilder
    func dismissable(_ dismiss:(()->Void)?) -> some View {
        if UIDevice.current.userInterfaceIdiom != .pad, let dismiss = dismiss{
            //在toolbar里增加按钮
            self.toolbar{
                //cancellationAction让按钮显示在左上角
                ToolbarItem(placement: .cancellationAction){
                    Button("关闭"){ dismiss()}//显示按钮并可调用闭包
                }
            }
        }else{
            self //不修改返回
        }
    }
}
