//
//  macOS.swift
//  EmojiArt
//
//  Created by CS193p Instructor on 5/26/21.
//

import SwiftUI

//共享区域的代码是用UIImage编写的,与NSImage非常相似,所以我们只需要typealias就能适配大部份时候
//但他们也有一点小区别，详细看后面
typealias UIImage = NSImage //将

//在macOS上，没有init(uiImage:)
extension Image {
    init(uiImage: UIImage) {// 我们可以提供uiImage初始化
        self.init(nsImage: uiImage)//改为init(nsImage:)
    }
}

//方便的变量将一个UIImage转换成一个Data
extension UIImage {
    //在macOS上，它将其转换为tiff而不是iOS里的jpeg
    var imageData: Data? { tiffRepresentation }
}


//在macOS上没有PaletteManager 因为它使用编辑模式(在macOS上不可用)
typealias PaletteManager = EmptyView


// 
struct Pasteboard {
    static var imageData: Data? {
        NSPasteboard.general.data(forType: .tiff) ?? NSPasteboard.general.data(forType: .png)
    }
    static var imageURL: URL? {
        (NSURL(from: NSPasteboard.general) as URL?)?.imageURL
    }
}

// L16 extensions to View for platform-specific modifications

extension View {
    // 在macOS里啥也不做，直接返回
    func wrappedInNavigationViewToMakeDismissable(_ dismiss: (() -> Void)?) -> some View {
        self
    }
    // 在macOS上调色板控制按钮需要稍微不同的样式，它还需要更大一点，以便它比滚动视图中的表情符号更大
    func paletteControlButtonStyle() -> some View {
        self.buttonStyle(PlainButtonStyle()).foregroundColor(.accentColor).padding(.vertical)
    }
    
    // 弹窗在macOS上没有水平填充，所以这将应用于所有的视图在弹出窗口中显示(可能这是不对的，但适用于演示)
    func popoverPadding() -> some View {
        self.padding(.horizontal)
    }
}


typealias Camera = CantDoItPhotoPicker
typealias PhotoLibrary = CantDoItPhotoPicker

// Camera和PhotoLibrary在Mac上不存在,创建一个"do nothing"视图,以便代码可以使用#if os(iOS)到处引用它们
//(在这个例子中是可执行的，因为isAvailable返回false)
struct CantDoItPhotoPicker: View {
    var handlePickedImage: (UIImage?) -> Void
    
    static let isAvailable = false
    
    var body: some View {
        EmptyView()
    }
}
